from typing import Union
from werkzeug.datastructures import FileStorage
from .validator import Validator, NoneValueException


class Nullable(Validator):
    def handler(self, value, field, request):
        if value is None or (isinstance(value, FileStorage) and value.content_type is None):
            raise NoneValueException(self.message)



import re
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class AlphaDash(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must only contain letters, numbers, dashes and underscores.'

    def handler(self, value, field, request):
        value = str(value)
        if not re.match('^[\w-]+$', value):
            raise ValidationError(self.message)



import abc
from typing import Any, Union


class ValidationError(ValueError):
    def __init__(self, message="", *args, **kwargs):
        ValueError.__init__(self, message, *args, **kwargs)


class StopValidation(Exception):
    def __init__(self, message="", *args, **kwargs):
        Exception.__init__(self, message, *args, **kwargs)


class NoneValueException(Exception):
    def __init__(self, message="", *args, **kwargs):
        Exception.__init__(self, message, *args, **kwargs)


class Validator(abc.ABC):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.data = {}
        self.parse = parse
        self.message = message or 'This field is invalid'

    def parse_data(self, value: Any) -> Any:
        return value

    @abc.abstractmethod
    def handler(self, value, field, request):
        pass

    def __call__(self, value, field, data, request):
        self.data = data
        self.field = field
        self.value = value
        self.request = request
        self.handler(self.value, self.field, self.request)
        return self.parse_data(self.value) if self.parse else self.value



from .validator import NoneValueException, Validator, ValidationError, StopValidation
from .accepted import Accepted
from .after import After
from .afterOrEqual import AfterOrEqual
from .alpha import Alpha
from .alphaDash import AlphaDash
from .alphaNum import AlphaNum
from .before import Before
from .beforeOrEqual import BeforeOrEqual
from .between import Between
from .boolean import Boolean
from .confirmed import Confirmed
from .currentPassword import CurrentPassword
from .date import Date
from .dateEquals import DateEquals
from .declined import Declined
from .different import Different
from .digits import Digits
from .digitsBetween import DigitsBetween
from .email import Email
from .exists import Exists
from .file import File
from .float import Float
from .inRule import In
from .integer import Integer
from .filled import Filled
from .gt import Gt
from .gte import Gte
from .max import Max
from .min import Min
from .notRegex import NotRegex
from .nullable import Nullable
from .regex import Regex
from .required import Required
from .size import Size
from .uuid import UUID
from .unique import Unique



from typing import Union
from werkzeug.datastructures import FileStorage
from .validator import Validator, ValidationError, StopValidation


class File(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be a file'

    def handler(self, value, field, request):
        if not isinstance(value, FileStorage):
            raise ValidationError(self.message)




from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Accepted(Validator):
    accepted = (True, 1, "yes", "on")

    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be accepted.'

    def parse_data(self, value):
        return value in self.accepted

    def handler(self, value, field, request):
        if value not in self.accepted:
            raise ValidationError(self.message)



import re
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Boolean(Validator):
    true_types = (True, "true", 1, "1",)
    false_types = (False, "false", 0, "0")

    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be true or false.'

    def parse_data(self, value):
        if value in self.true_types:
            return True
        elif value in self.false_types:
            return False
        else:
            return value

    def handler(self, value, field, request):
        if not value in self.true_types and not value in self.false_types:
            raise ValidationError(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Integer(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be an integer.'

    def parse_data(self, value):
        return int(value)

    def handler(self, value, field, request):
        try:
            assert not isinstance(value, float)
            value = int(value)
        except:
            raise StopValidation(self.message)



import re
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Between(Validator):
    number_types = (int, float)

    def __init__(self, min: int, max: int, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is invalid'
        self.min = min
        self.max = max

    def handler(self, value, field, request):
        has_error = False
        if isinstance(value, str) or isinstance(value, list):
            has_error = len(value) < self.min or len(value) > self.max
        elif type(value) in self.number_types:
            has_error = value < self.min or value > self.max

        if has_error:
            raise ValidationError(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Exists(Validator):
    def __init__(self, model,
                 column: Union[str, None] = None,
                 message: Union[str, None] = None,
                 parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This attribute is invalid'
        self.model = model
        self.column = column

    def handler(self, value, field, request):
        column = self.column or field
        filters = [getattr(self.model, column) == value]
        exists = self.model.query.filter(*filters).count()
        if not exists:
            raise ValidationError(self.message)




from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Required(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is required'

    def handler(self, value, field, request):
        if value is None or len(value) == 0:
            raise StopValidation(self.message)



import abc

from .filters import strip_filter
from .validators import (
    NoneValueException,
    StopValidation,
    ValidationError
)


class FormRequest(abc.ABC):
    def __init__(self, request) -> None:
        self._data = {}
        self._errors = {}
        self._filters = [strip_filter]
        self._request = request
        self.load_data()
        self.apply_filters()

    @property
    def data(self):
        return self._data

    @property
    def errors(self):
        return self._errors

    @property
    def request(self):
        return self._request

    def load_data(self):
        json_data = self._request.json or {}
        get_post_data = self._request.values.to_dict()
        files_data = self._request.files.to_dict()

        self._data = {**json_data, **get_post_data, **files_data}

    def apply_filters(self):
        for k, v in self._data.items():
            for f in self._filters:
                v = f(v)
            self._data[k] = v

    @abc.abstractmethod
    def rules(self):
        pass

    def validate(self):
        rules = self.rules()
        for field, validators in rules.items():
            errors = []
            for validator in validators:
                try:
                    new_val = validator(self._data.get(field), field, self.data, self.request)
                    self._data[field] = new_val
                except StopValidation as err:
                    errors.append(str(err))
                    break
                except ValidationError as err:
                    errors.append(str(err))
                except NoneValueException:
                    break
            if len(errors) > 0:
                self.errors[field] = errors
        return not bool(self.errors)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Different(Validator):
    def __init__(self, field: str, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is not valid.'
        self.compare_field = field

    def handler(self, value, field, request):
        another_field_value = self.data.get(self.compare_field)

        if value == another_field_value:
            raise ValidationError(self.message)



from collections.abc import Iterable
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Filled(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must have a value.'

    def handler(self, value, field, request):
        if value is None \
            or (
                (isinstance(value, str) or isinstance(value, Iterable)) \
                and len(value) == 0
            ):
            raise StopValidation(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class In(Validator):
    def __init__(self, accepted_values = Union[list, tuple], message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is not valid.'
        self.accepted_values = accepted_values

    def handler(self, value, field, request):
        if value not in self.accepted_values:
            raise ValidationError(self.message)



from typing import Union
from collections.abc import Iterable
from .validator import Validator, ValidationError, StopValidation


class Gt(Validator):
    def __init__(self, length: int, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.size = length
        self.message = message or 'This field is not valid.'

    def handler(self, value, field, request):
        if isinstance(value, int) or isinstance(value, float):
            value = str(value)

        # TODO: implement support for files

        if len(value) <= self.size:
            raise ValidationError(self.message)



from typing import Any, Union
from .validator import Validator, ValidationError, StopValidation


class DigitsBetween(Validator):
    def __init__(self, min: int, max: int, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is not valid.'
        self.min = min
        self.max = max

    def parse_data(self, value: Any) -> int:
        return int(value)

    def handler(self, value, field, request):
        len_value = len(value)
        if not value.isnumeric() or len_value < self.min or len_value > self.max:
            raise ValidationError(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class CurrentPassword(Validator):
    def __init__(self, model, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'The password is incorrect.'
        self.model = model

    def handler(self, value, field, request):
        if not self.model.verify_password(value):
            raise ValidationError(self.message)



from datetime import datetime
from typing import Union
from .date import Date
from .validator import Validator, ValidationError, StopValidation


class After(Validator):
    def __init__(self, date, format='%Y-%m-%d %H:%M:%S', message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This attribute is not valid'
        self.format = format
        self.date = date

    def parse_data(self, value):
        return self.dt_formated

    def handler(self, value, field, request):
        date_validator = Date(self.format, self.message)
        self.dt_formated = date_validator(value, field, self.data, request)

        if self.dt_formated <= self.date:
            raise ValidationError(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Digits(Validator):
    def __init__(self, length: int, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is not valid.'
        self.length = length

    def parse_data(self, value):
        return int(value)

    def handler(self, value, field, request):
        if not value.isnumeric() or len(value) != self.length:
            raise ValidationError(self.message)



from email_validator import validate_email, EmailNotValidError
from typing import Any, Union
from .validator import Validator, ValidationError, StopValidation


class Email(Validator):
    def __init__(self, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be a valid email address.'

    def handler(self, value, field, request):
        try:
            valid = validate_email(value)
            self.value = valid.email
        except EmailNotValidError as e:
            raise ValidationError(self.message)



import re
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class NotRegex(Validator):
    def __init__(self, pattern: str, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.pattern = pattern
        self.message = message or 'This field is not valid.'

    def handler(self, value, field, request):
        if re.match(self.pattern, value):
            raise ValidationError(self.message)



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="flask-FormRequest",
    version="0.0.24",
    author="edcilo",
    description="Request validator for flask",
    long_description=long_description,
    long_description_content_type="text/markdown",
    exclude=["test"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    include_package_data=True,
    packages=['flaskFormRequest', 'flaskFormRequest.filters', 'flaskFormRequest.validators'],
    package_dir={'':'src'},
    install_requires=[
        'email-validator==1.1.3',
    ],
)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Unique(Validator):
    def __init__(self,
                 model,
                 column: Union[str, None] = None,
                 except_id: Union[str, int, None] = None,
                 message: Union[str, None] = None,
                 parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field is not valid.'
        self.model = model
        self.column = column
        self.except_id = except_id

    def handler(self, value, field, request):
        column = self.column or field

        filters = [getattr(self.model, column) == value]
        if self.except_id is not None:
            filters.append(self.model.id != self.except_id)

        exists = self.model.query.filter(*filters).count()

        if exists:
            raise ValidationError(self.message)



from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Confirmed(Validator):
    def __init__(self, suffix: str = '_confirmation', message: Union[str, None] = None, parse: bool = True) -> None:
        self.suffix = suffix
        self.parse = parse
        self.message = message or 'This field confirmation does not match.'

    def handler(self, value, field, request):
        field_to_match = self.data.get(f'{field}{self.suffix}')
        if value != field_to_match:
            raise ValidationError(self.message)



from datetime import datetime
from typing import Union
from .validator import Validator, ValidationError, StopValidation


class Date(Validator):
    def __init__(self, format='%Y-%m-%d', message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This attribute is not a valid date.'
        self.format = format

    def parse_data(self, value):
        return self.dt_formated

    def handler(self, value, field, request):
        try:
            self.dt_formated = datetime.strptime(value, self.format)
        except:
            raise ValidationError(self.message)



from typing import Union
from uuid import UUID as UUIDHelper
from .validator import Validator, ValidationError, StopValidation


class UUID(Validator):
    def __init__(self, version: int = 4, message: Union[str, None] = None, parse: bool = True) -> None:
        self.parse = parse
        self.message = message or 'This field must be a valid UUID'
        self.version = version

    def parse_data(self, value):
        return UUIDHelper(value, version=self.version)

    def handler(self, value, field, request):
        try:
            value = UUIDHelper(value, version=self.version)
        except ValueError:
            raise StopValidation(self.message)
